/*================================================================
  ===     EDITIONS TRANSOCEANIC  -  ELECTRONIQUE PRATIQUE      ===
  ===                (c) Yves MERGY 01/2011                    ===
  ================================================================
  ===  PROGRAMME POUR LE ROBOT ARDUINO-EP QUI SAIT SE REPERER  ===
  ================================================================ */

//============================== AJOUT DE LIBRAIRIES ADDITIONNELLES
#include <Wire.h>    // Gestion du protocole I2C
#include <CMPS03.h>  // Gestion du module CMPS03 (boussole)
#include <Servo.h>   // Gestion des servomoteurs
  
//============================== DECLARATION DE LA VARIABLE "LCD" DE TYPE "LiquidCrystal"
  Servo SERVOD;    // Création d'un objet "Servo" 
  Servo SERVOC;    // Création d'un objet "Servo" 
  Servo SERVOG;    // Création d'un objet "Servo" 
  CMPS03 BOUSSOLE; // Création d'un objet "CMPS03"
  
//============================== DECLARATION DES VARIABLES ATTRIBUEES AUX BROCHES D'E/S
  int BUZ = 2;      // Buzzer piezzo sur Port 2
  int LEDG = 9;     // Led GAUCHE sur Port 9
  int LEDD = 10;    // Led DROITE sur Port 10
  int TAR = 14;     // Touche ARRIERE sur Port 14
  int TCE = 15;     // Touche CENTRALE sur Port 15
  int TAV = 16;     // Touche AVANT sur Port 16
  int IR = 3;       // Capteur TELEMETRIQUE sur Port 17 (ANA3)

//============================== DECLARATION DES VARIABLES DU PROGRAMME
  int TI;           // Variable temporaire
  int DISTANCE;     // Valeur de lecture du capteur télémétrique 
  int ORIENTATION;  // Valeur de lecture de la boussole
  int REGA;         // Regard devant et valeur de l'orientation
  int OBSA;         // Détection d'un obstacle devant et valeur de la distance
  int REGG;         // Regard à gauche et valeur de l'orientation
  int REGD;         // Regard à droite et valeur de l'orientation
  int OBSG;         // Détection d'un obstacle à gauche et valeur de la distance
  int OBSD;         // Détection d'un obstacle à droite et valeur de la distance
  int VIG1;         // Mémorisation 1 de l'orientation à gauche
  int VIG2;         // Mémorisation 2 de l'orientation à gauche
  int VIAV;         // Mémorisation de l'orientation vers l'avant
  int VID1;         // Mémorisation 1 de l'orientation à droite
  int VID2;         // Mémorisation 2 de l'orientation à droite
  int MODE;         // Mode de déplacement (0=Repos 1=avec boussole 2=sans boussole)
  int CPT1;         // Compteur numéro 1
  int NEUTRE;       // Valeur du NEUTRE

//============================== PROCEDURE D'INITIALISATION OBLIGATOIRE NOMMEE "setup"
void setup() {
//  Serial.begin(9600);
  Wire.begin();          //Initialisation du dialogue I2C
  SERVOD.attach(3);      // Servomoteur DROIT sur Port 3 (PWM)
  SERVOC.attach(5);      // Servomoteur CENTRAL sur Port 5 (PWM)
  SERVOG.attach(6);      // Servomoteur GAUCHE sur Port 6 (PWM)
  pinMode(BUZ, OUTPUT);  // Configuration de la broche du BUZZER en sortie
  pinMode(LEDG, OUTPUT); // Configuration de la broche de la LED GAUCHE en sortie
  pinMode(LEDD, OUTPUT); // Configuration de la broche de la LED DROITE en sortie
  pinMode(TAR, INPUT);   // Configuration de la broche de la TOUCHE ARRIERE en entrée
  pinMode(TCE, INPUT);   // Configuration de la broche de la TOUCHE CENTRALE en entrée
  pinMode(TAV, INPUT);   // Configuration de la broche de la TOUCHE AVANT en entrée
  pinMode(IR, INPUT);    // Configuration de la broche du capteur TELEMETRIQUE en entrée
  NEUTRE=90;             // Valeur du NEUTRE fixée à 90 degrés
  ARRET();               // Robot à l'arrêt
  tone (BUZ,2000,600);   // Bip sonore à 2kHz de 0,2 seconde
  delay(60);            // Pause de 0,2 seconde
  tone (BUZ,1000,200);   // Bip sonore à 2kHz de 0,2 seconde
  delay(200);            // Pause de 0,2 seconde
  tone (BUZ,3000,400);   // Bip sonore à 2kHz de 0,2 seconde
  delay(400);            // Pause de 0,2 seconde
  tone (BUZ,2000,200);   // Bip sonore à 2kHz de 0,2 seconde
  delay(200);            // Pause de 0,2 seconde
  tone (BUZ,1000,200);   // Bip sonore à 2kHz de 0,2 seconde
  delay(200);            // Pause de 0,2 seconde
  tone (BUZ,500,600);   // Bip sonore à 2kHz de 0,2 seconde
  delay(600);            // Pause de 0,2 seconde
  MODE=0;                // Mode de fonctionnement fixé à 0 (repos et attente)
  CPT1=0;                // Initialisation du compteur
  delay(200);            // Pause de 0,2 seconde
}

//============================== BOUCLE PRINCIPALE OBLIGATOIRE NOMMEE "loop"
void loop() {
  if (digitalRead(TCE) == LOW) MODE=0; // Test de la touche CENTRALE
  if (digitalRead(TAV) == LOW) MODE=1; // Test de la touche AVANT 
  if (digitalRead(TAR) == LOW) MODE=2; // Test de la touche ARRIERE
  if (MODE==0) REPOS();                // Repos si Touche CE appuyée
  if (MODE==1) COMPLEXE();             // Déplacements avec boussole si Touche AV appuyée 
  if (MODE==2) SIMPLE();               // Déplacements sans boussole si Touche AR appuyée
}

//==================== MODE 0 : ROBOT A L'ARRET, ATTENTE D'UNE ACTION ==========
//====================          ET MANIFESTATIONS DE PRESENCE         ==========
void REPOS() {
  ARRET();
  CPT1++;                     // Compteur incrémenté
  delay(10);                  // Pause de base pour le comptage (10 millisecondes)
  DISTANCE = analogRead(IR);  // Lecture de la valeur du capteur TELEMETRIQUE
  if (DISTANCE >= 100) tone (BUZ,DISTANCE*6,100); // Bruit variable selon distance
  if (CPT1>=1000) CPT1=0;     // Initialisation du compteur si le maxi est atteint
  if (CPT1==100) {  // Si compteur=100 => Servo à gauche et bruit
    SERVOC.write(70);
    tone (BUZ,500,100);
  }
  if (CPT1==270) {  // Si compteur=270 => Servo à droite
    SERVOC.write(110);
  }
  if (CPT1==400) {  // Si compteur=400 => Servo au centre
    SERVOC.write(NEUTRE);
  }
  if (CPT1<200) {  // Si compteur plus bas que 200 => Leds G et D éteintes
    digitalWrite(LEDG, LOW);   
    digitalWrite(LEDD, LOW);   
  }
  if (CPT1>=200 && CPT1>=250) {  // Si compteur entre 200 et 250 => Led G allumée
    digitalWrite(LEDG, HIGH);   
  }
  if (CPT1>250 && CPT1>=300) {  // Si compteur entre 251 et 300 => Led D allumée et G éteinte
    digitalWrite(LEDD, HIGH);   
    digitalWrite(LEDG, LOW);   
  }
  if (CPT1>300 && CPT1>=400) {  // Si compteur entre 301 et 400 => Led D éteinte
    digitalWrite(LEDD, LOW);   
  }
  if (CPT1>400) {  // Si compteur plus haut que 400 => Leds G et D allumées
    digitalWrite(LEDG, HIGH);   
    digitalWrite(LEDD, HIGH);   
  }
}
//==================== MODE 1 : DEPLACEMENTS AVEC LA BOUSSOLE ==========
void COMPLEXE() {
  digitalWrite(LEDG, LOW);  // Led GAUCHE éteinte
  digitalWrite(LEDD, LOW);  // Led DROITE éteinte 
  AVANT();  // Roule en AVANT
  if (digitalRead(TCE) == LOW) {  // Test si touche CENTRALE appuyée
    MODE=0;   // Repos si touche CE appuyée
    return;   // et sortie forcée de la procédure
  }
  do {  // Lecture du TELEMETRE tant qu'il n'y a pas d'obstacle en vue
    DISTANCE = analogRead(IR);    
    if (digitalRead(TCE) == LOW) {  // Test si touche CE appuyée => Repos forcé
      MODE=0;
      return;
    }
  } while(DISTANCE<=200);
  ARRET();    // Robot à l'ARRET
  delay(200); // Pause de 0,2 seconde
  ARRIERE();  // Roule en AVANT
  delay(500); // Pause de 0,5 seconde
  ARRET();    // Robot à l'ARRET
  delay(200); // Pause de 0,2 seconde
  CHERCHE();  // Procédure de recherche pour éviter l'obstacle
  delay(100); // Pause de 0,1 seconde
  if (OBSG>=OBSD) {  // Si l'obstacle est à gauche
    REGA=REGD;       // Valeur de la direction à prendre vers la droite
    DROITE();        // Virage vers la DROITE
  }
  if (OBSD>=OBSG) {  // Si l'obstacle est à droite
    REGA=REGG;       // Valeur de la direction à prendre vers la gauche
    GAUCHE();        // Virage vers la GAUCHE
  }
  do {  // Le robot vire à G ou à D ....
    ORIENTATION = (BOUSSOLE.read());             // Lecture de la direction de la boussole
    ORIENTATION = map(ORIENTATION,0,3598,0,72);  // Adaptation du format pour les déplacements
    delay(50);                                   // Pause de 50 millisecondes
    if (digitalRead(TCE) == LOW) {               // Test si touche CE appuyée => Repos forcé
      MODE=0;
      return;
    }
  } while(ORIENTATION!=REGA);  // .... Tant que la direction exacte n'est pas obtenue
  ARRET();    // Robot à l'ARRET
  delay(500); // Pause de 0,5 seconde
}
//==================== MODE 2 : DEPLACEMENTS SANS LA BOUSSOLE ==========
void SIMPLE() {
  digitalWrite(LEDG, LOW);  // Led GAUCHE éteinte
  digitalWrite(LEDD, LOW);  // Led DROITE éteinte
  AVANT();  // Roule en AVANT
  if (digitalRead(TCE) == LOW) {  // Test si touche CE appuyée => Repos forcé
    MODE=0;
    return;
  }
  VISIOND();    // Procédure de regard vers la DROITE
  if(REGD>50){  // Si obstacle en vue à droite
    do {  // Boucle d'évitement
      GAUCHE();   // Virage à GAUCHE pour retrouver la voie libre
      delay(500); // Pendant 0,5 seconde
      DISTANCE = analogRead(IR);  // Lecture du TELEMETRE ....
      delay(10);            
      if (digitalRead(TCE) == LOW) {  // Test si touche CE appuyée => Repos forcé
        MODE=0;
        return;
      }
    } while(DISTANCE>50);  // .... Tant que l'obstacle est en vue
    delay(50);
    ARRET(); // Arrêt
    delay(200);
    AVANT(); // Roule en avant
  }

  VISIONG();    // Procédure de regard vers la GAUCHE
  if(REGG>50){  // Si obstacle en vue à gauche
    do {  // Boucle d'évitement
      DROITE(); // Virage à DROITE pour retrouver la voie libre
      delay(500); // Pendant 0,5 seconde
      DISTANCE = analogRead(IR);  // Lecture du TELEMETRE ....
      delay(10);            
      if (digitalRead(TCE) == LOW) {  // Test si touche CE appuyée => Repos forcé
        MODE=0;
        return;
      }
    } while(DISTANCE>50);  // .... Tant que l'obstacle est en vue
    delay(50);
    ARRET(); // Arrêt
    delay(200);
    AVANT(); // Roule en avant
  }
  
  SERVOC.write(NEUTRE); // Regarde en FACE
  delay(300);
  DISTANCE = analogRead(IR);  // Lecture du TELEMETRE
  delay(10);
  if(DISTANCE>200) {  // Arrêt en cas d'obstacle
    ARRET();
    delay(200);
    ARRIERE(); // Arrière pendant 2 S
    delay(2000);
    ARRET();  // Arrêt
    delay(200);
    VISIOND();      // Procédure de regard vers la DROITE
    VISIONG();      // Procédure de regard vers la GAUCHE
    if(REGD<REGG){  // Si obstacle en vue à gauche
      DROITE();     // Virage à DROITE pour retrouver la voie libre
      do {  // Boucle d'évitement
        DISTANCE = analogRead(IR);  // Lecture du TELEMETRE ....
        delay(10);            
        if (digitalRead(TCE) == LOW) {  // Test si touche CE appuyée => Repos forcé
          MODE=0;
          return;
        }
      } while(DISTANCE>50);  // .... Tant que l'obstacle est en vue
      delay(50);
      ARRET(); // Arrêt
      delay(200);
      AVANT(); // Roule en avant
    }
    if(REGG<REGD){  // Si obstacle en vue à droite
      GAUCHE();     // Virage à GAUCHE pour retrouver la voie libre
      do {  // Boucle d'évitement
        DISTANCE = analogRead(IR);  // Lecture du TELEMETRE ....
        delay(10);            
        if (digitalRead(TCE) == LOW) {  // Test si touche CE appuyée => Repos forcé
          MODE=0;
          return;
        }
      } while(DISTANCE>50);  // .... Tant que l'obstacle est en vue
      delay(50);
      ARRET(); // Arrêt
      delay(200);
      AVANT(); // Roule en avant
    }
  }
}
//==================== VIRAGE A GAUCHE ==========
void GAUCHE() {
  digitalWrite(LEDG, HIGH);   
  digitalWrite(LEDD, LOW);   
  SERVOD.write(85);
  SERVOG.write(85);  
}
//==================== VIRAGE A DROITE ==========
void DROITE() {
  digitalWrite(LEDG, LOW);   
  digitalWrite(LEDD, HIGH);   
  SERVOD.write(95);
  SERVOG.write(95);  
}
//==================== ROULE EN AVANT ==========
void AVANT() {
  digitalWrite(LEDG, HIGH);   
  digitalWrite(LEDD, HIGH);   
  SERVOD.write(0);
  SERVOG.write(180);  
}
//==================== ROULE EN ARRIERE ==========
void ARRIERE() {
  SERVOD.write(180);
  SERVOG.write(0);  
}
//==================== ARRET ==========
void ARRET() {
  SERVOD.write(NEUTRE);
  SERVOG.write(NEUTRE);  
}
//==================== REGARDE A DROITE ==========
void VISIOND() {
  SERVOC.write(20); // Regarde à droite
  delay(500);
  DISTANCE = analogRead(IR);
  delay(10);
  REGD=DISTANCE;
  SERVOC.write(NEUTRE); // Regarde en face
  delay(500);
}
//==================== REGARDE A GAUCHE ==========
void VISIONG() {
  SERVOC.write(160); // Regarde à droite
  delay(500);
  DISTANCE = analogRead(IR);
  delay(10);
  REGG=DISTANCE;
  SERVOC.write(NEUTRE); // Regarde en face
  delay(500);
}
//==================== RECHERCHE DE LA DIRECTION A PRENDRE EN FONCTION DE L'OBSTACLE EN VUE ==========
void CHERCHE() {
  OBSG=0;  // Initialisation de la valeur
  OBSD=0;  // Initialisation de la valeur
  ARRET(); // Robot à l'arrêt
  ORIENTATION = (BOUSSOLE.read());             // Lecture de la boussole
  delay(50);          // Pause de traitement
  ORIENTATION = map(ORIENTATION,0,3598,0,72);  // Adaptation du format
  REGA=ORIENTATION;   // Mémorisation dans la variable REGA
  SERVOC.write(170);  // Rotation de la tête
  delay(500);
  ORIENTATION = (BOUSSOLE.read());             // Lecture de la boussole
  delay(50);          // Pause de traitement
  ORIENTATION = map(ORIENTATION,0,3598,0,72);  // Adaptation du format
  DISTANCE = analogRead(IR);  // Lecture du TELEMETRE
  delay(50);          // Pause de traitement
  if (DISTANCE>OBSG) {  // Mémorisation si obstacle en vue
    OBSG=DISTANCE;
    REGG=ORIENTATION;    
  }
  SERVOC.write(NEUTRE);  // Tête en face
  delay(500);
  SERVOC.write(10);  // Rotation de la tête dans l'autre sens
  delay(500);
  ORIENTATION = (BOUSSOLE.read());             // Lecture de la boussole
  delay(50);          // Pause de traitement
  ORIENTATION = map(ORIENTATION,0,3598,0,72);  // Adaptation du format
  DISTANCE = analogRead(IR);  // Lecture du TELEMETRE
  delay(50);          // Pause de traitement
  if (DISTANCE>OBSD) {  // Mémorisation si obstacle en vue
    OBSD=DISTANCE;
    REGD=ORIENTATION;    
  }
  SERVOC.write(NEUTRE);  // Rotation de la tête
  delay(500);
}

